<!DOCTYPE html>
<html>
{% include 'head.html' %}
<body>
    <div id="centered-container">
        {% include 'header.html' %}
        {% include 'sign-out.html' %}
        <div id="start-text">
            <p>
                <a href="/" class="link-button">
                    <i class="fas fa-comments"></i></i> Thank you! Let's talk!
                </a>
            </p>
        </div>
        {% include 'footer.html' %}
    </div>
</body>
</html>
