<!DOCTYPE html>
<html>
<head>
<script type="module" src="/public/search-widget.js"></script>
<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/5.9.0/css/all.min.css" integrity="sha512-q3eWabyZPc1XTCmF+8/LuE1ozpg5xxn7iO89yfSOd5/oKvyqLngoNGsx8jq92Y8eXJ/IRxQbEC+FGSYxtk2oiw==" crossorigin="anonymous" referrerpolicy="no-referrer" />
</head>
<body>
<h1> This is a header </h1>
<search-widget endpoint="/public/search"></search-widget>
<p>This is some text</p>
</body>
</html>
